@@ -38,6 +38,7 @@
 enum {
 	BOGUS,
 	APPLY,
+	RESTORE,
 	COMPLEXITY_CHANGE,
 	LAST_SIGNAL
 };
@@ -62,8 +63,8 @@ gst_dialog_get_widget (GstDialog *xd, const char *widget)
 	return w;
 }
 
-static void
-apply_widget_policies (GstDialog *xd)
+void
+gst_dialog_apply_widget_policies (GstDialog *xd)
 {
 	GSList *list;
 
@@ -115,7 +116,7 @@ gst_dialog_set_complexity (GstDialog *xd, GstDialogComplexity c)
 	else
 		gst_conf_set_boolean (xd->tool, "advanced_mode", FALSE);
 
-	apply_widget_policies (xd);
+	gst_dialog_apply_widget_policies (xd);
 
 	/* set the complexity button appearance */
 	gtk_label_set_text (GTK_LABEL (xd->complexity_button_label), _(label[c]));
@@ -130,11 +131,10 @@ gst_dialog_freeze (GstDialog *xd)
 {
 	g_return_if_fail (xd != NULL);
 	g_return_if_fail (GST_IS_DIALOG (xd));
-	g_return_if_fail (xd->frozen >= 0);
 
 	d(g_message ("freezing %p", xd));
 
-	xd->frozen++;
+	xd->frozen = TRUE;
 }
 
 void 
@@ -142,11 +142,10 @@ gst_dialog_thaw (GstDialog *xd)
 {
 	g_return_if_fail (xd != NULL);
 	g_return_if_fail (GST_IS_DIALOG (xd));
-	g_return_if_fail (xd->frozen >= 0);
 
 	d(g_message ("thawing %p", xd));
 
-	xd->frozen--;
+	xd->frozen = FALSE;
 }
 
 void
@@ -181,7 +180,7 @@ gst_dialog_get_modified (GstDialog *xd)
 	g_return_val_if_fail (xd != NULL, FALSE);
 	g_return_val_if_fail (GST_IS_DIALOG (xd), FALSE);
 
-	return GTK_WIDGET_SENSITIVE (xd->apply_button);
+	return xd->is_modified;
 }
 
 void
@@ -190,7 +189,7 @@ gst_dialog_set_modified (GstDialog *xd, gboolean state)
 	g_return_if_fail (xd != NULL);
 	g_return_if_fail (GST_IS_DIALOG (xd));
 
-	gtk_widget_set_sensitive (xd->apply_button, state);
+	xd->is_modified = state;
 }
 
 void
@@ -227,7 +226,6 @@ gst_dialog_class_init (GstDialogClass *klass)
 	object_class = (GtkObjectClass *)klass;
 	parent_class = gtk_type_class (GNOME_TYPE_APP);
 
-#if 1	
 	gstdialog_signals[APPLY] = 
 		g_signal_new ("apply",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -237,6 +235,15 @@ gst_dialog_class_init (GstDialogClass *klass)
 			      gst_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+	gstdialog_signals[RESTORE] =
+		g_signal_new ("restore",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GstDialogClass, restore),
+			      NULL, NULL,
+			      gst_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
 	
 	gstdialog_signals[COMPLEXITY_CHANGE] =
 		g_signal_new ("complexity_change",
@@ -247,27 +254,6 @@ gst_dialog_class_init (GstDialogClass *klass)
 			      gst_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
-#else	
-	gstdialog_signals[APPLY] = 
-		gtk_signal_new ("apply",
-				GTK_RUN_LAST,
-				object_class->type,
-				GTK_SIGNAL_OFFSET (GstDialogClass, apply),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
-
-	gstdialog_signals[COMPLEXITY_CHANGE] =
-		gtk_signal_new ("complexity_change",
-				GTK_RUN_LAST,
-				object_class->type,
-				GTK_SIGNAL_OFFSET (GstDialogClass, complexity_change),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
-#endif	
-
-#if 0
-	gtk_object_class_add_signals (object_class, gstdialog_signals, LAST_SIGNAL);
-#endif	
 
 	object_class->destroy = gst_dialog_destroy;
 }
@@ -337,10 +323,9 @@ gst_dialog_set_widget_policies (GstDialog *xd, const GstWidgetPolicy *xwp)
 	GstWidget *xw;
 	int i;
 
-	for (i = 0; xwp [i].widget; i++)
+	for (i = 0; xwp [i].widget; i++) {
 		xw = gst_widget_new (xd, xwp [i]);
-
-	apply_widget_policies (xd);
+	}
 }
 
 void
@@ -361,7 +346,7 @@ gst_dialog_set_widget_user_modes (GstDialog *xd, const GstWidgetUserPolicy *xwup
 		gst_widget_set_user_mode (xw, xwup [i].mode);
 	}
 
-	apply_widget_policies (xd);
+	gst_dialog_apply_widget_policies (xd);
 }
 
 GstWidget *
@@ -444,7 +429,20 @@ complexity_cb (GtkWidget *w, gpointer data)
 }
 
 static void
-apply_cb (GtkWidget *w, gpointer data)
+restore_config (gpointer data)
+{
+	GstDialog *dialog;
+
+	g_return_if_fail (data != NULL);
+	g_return_if_fail (GST_IS_DIALOG (data));
+
+	dialog = GST_DIALOG (data);
+
+	g_signal_emit (G_OBJECT (dialog), gstdialog_signals[RESTORE], 0);
+}
+
+static void
+apply_config (gpointer data)
 {
 	GstDialog *dialog;
 
@@ -457,8 +455,32 @@ apply_cb (GtkWidget *w, gpointer data)
 		return;
 
 	g_signal_emit (G_OBJECT (dialog), gstdialog_signals[APPLY], 0);
+}
 
-	gst_dialog_set_modified (dialog, FALSE);
+void
+gst_dialog_ask_apply (GstDialog *dialog)
+{
+        g_return_if_fail (dialog != NULL);
+        g_return_if_fail (GST_IS_DIALOG (dialog));
+
+        if (gst_dialog_get_modified (dialog)) {
+                /* Changes have been made. */
+                GtkWidget *w;
+                gint       res;
+                                                                                
+                w = gtk_message_dialog_new (GTK_WINDOW (dialog),
+                                            GTK_DIALOG_MODAL,
+                                            GTK_MESSAGE_QUESTION,
+                                            GTK_BUTTONS_YES_NO,
+                                            _("There are changes which haven't been applied.\n"
+                                              "Apply them now?"));
+                                                                                
+                res = gtk_dialog_run (GTK_DIALOG (w));
+                gtk_widget_destroy (w);
+                                                                                
+                if (res == GTK_RESPONSE_YES)
+                        apply_config (dialog);
+        }
 }
 
 static void
@@ -467,25 +489,6 @@ dialog_close (GstDialog *dialog)
 	g_return_if_fail (dialog != NULL);
 	g_return_if_fail (GST_IS_DIALOG (dialog));
 
-	if (gst_dialog_get_modified (dialog)) {
-		/* Changes have been made. */
-		GtkWidget *w;
-		gint       res;
-
-		w = gtk_message_dialog_new (GTK_WINDOW (dialog),
-					    GTK_DIALOG_MODAL,
-					    GTK_MESSAGE_QUESTION,
-					    GTK_BUTTONS_YES_NO,
-					    _("There are changes which haven't been applied.\n"
-					      "Apply them now?"));
-
-		res = gtk_dialog_run (GTK_DIALOG (w));
-		gtk_widget_destroy (w);
-
-		if (res == GTK_RESPONSE_YES)
-			apply_cb (NULL, dialog);
-	}
-
 	gtk_widget_hide (GTK_WIDGET (dialog));
 
 	if (dialog == dialog->tool->main_dialog)
@@ -499,17 +502,8 @@ dialog_delete_event_cb (GtkWidget *w, GdkEvent *event, gpointer data)
 }
 
 static void
-close_cb (GtkWidget *w, gpointer data)
+apply_cb (GtkWidget *w, gpointer data)
 {
-	dialog_close (data);
-}
-
-static void
-help_cb (GtkWidget *w, gpointer data)
-{
-#warning FIXME	
-#if 0
-	GnomeHelpMenuEntry help_entry = { NULL, "index.html" };
 	GstDialog *dialog;
 
 	g_return_if_fail (data != NULL);
@@ -517,14 +511,45 @@ help_cb (GtkWidget *w, gpointer data)
 
 	dialog = GST_DIALOG (data);
 
-	help_entry.name = g_strdup_printf ("%s-admin", dialog->tool->name);
+	if (gst_dialog_get_modified (dialog))
+		apply_config (dialog);
+}
 
-	gnome_help_display (NULL, &help_entry);
+static void
+cancel_cb (GtkWidget *w, gpointer data)
+{
+	GstDialog *dialog;
 
-	g_free (help_entry.name);
-#else
-	g_warning ("FIXME ! Port help_cb to 2.0\n");
-#endif
+	g_return_if_fail (data!= NULL);
+	g_return_if_fail (GST_IS_DIALOG (data));
+
+	dialog = GST_DIALOG (data);
+
+	if (gst_dialog_get_modified (dialog))
+		restore_config (dialog);
+
+	dialog_close (dialog);
+}
+
+static void
+accept_cb (GtkWidget *w, gpointer data)
+{
+	GstDialog *dialog;
+	
+	g_return_if_fail (data != NULL);
+	g_return_if_fail (GST_IS_DIALOG (data));
+
+	dialog = GST_DIALOG (data);
+
+	if (gst_dialog_get_modified (dialog))
+		apply_config (dialog);
+	
+	dialog_close (dialog);
+}
+
+static void
+help_cb (GtkWidget *w, gpointer data)
+{
 }
 
 void
@@ -612,7 +637,7 @@ gst_dialog_construct (GstDialog *dialog, GstTool *tool,
 
 	if (GTK_WIDGET_TOPLEVEL (dialog->child)) {
 		g_error ("The widget \"%s\" should not be a toplevel widget in the .glade file\n"
-			 "You just need to add the widget inside a GtkWindow so that it can be deparented it.", widget);
+			 "You just need to add the widget inside a GtkWindow so that it can be deparented.", widget);
 	}
 
 	gtk_widget_ref (dialog->child);
@@ -637,11 +662,11 @@ gst_dialog_construct (GstDialog *dialog, GstTool *tool,
 
 	w = glade_xml_get_widget (xml, "apply");
 	g_signal_connect (G_OBJECT (w), "clicked", G_CALLBACK (apply_cb), dialog);
+	w = glade_xml_get_widget (xml, "cancel");
+	g_signal_connect (G_OBJECT (w), "clicked", G_CALLBACK (cancel_cb), dialog);
 
-	dialog->apply_button = w;
-
-	w = glade_xml_get_widget (xml, "close");
-	g_signal_connect (GTK_OBJECT (w), "clicked", G_CALLBACK (close_cb), dialog);
+	w = glade_xml_get_widget (xml, "accept");
+	g_signal_connect (GTK_OBJECT (w), "clicked", G_CALLBACK (accept_cb), dialog);
 	
 	gst_dialog_set_modified (dialog, FALSE);
 	gtk_widget_hide (dialog->complexity_button);
@@ -653,7 +678,9 @@ gst_dialog_construct (GstDialog *dialog, GstTool *tool,
 
 	gst_dialog_set_complexity (dialog, val);
 
-	g_signal_connect (GTK_OBJECT (dialog), "delete_event", G_CALLBACK (dialog_delete_event_cb), dialog);
+	g_signal_connect (G_OBJECT (dialog), "delete_event", G_CALLBACK (dialog_delete_event_cb), dialog);
+
+	g_signal_connect (G_OBJECT (tool->remote_dialog), "delete_event", G_CALLBACK (dialog_delete_event_cb), dialog);
 
 }
 
