@@ -55,11 +55,12 @@ struct _GstDialog {
 	GtkWidget *child;
 
 	/* Common widgets */
-	GtkWidget *apply_button;
 	GtkWidget *complexity_button;
 	GtkWidget *complexity_button_label;
 	GtkWidget *complexity_button_image;
 
+	gboolean is_modified;
+
 	GstDialogComplexity complexity;
 	gboolean frozen;
 
@@ -71,6 +72,7 @@ struct _GstDialogClass {
 	GnomeAppClass parent_class;
 
 	void (*apply)             (GstDialog *);
+	void (*restore)           (GstDialog *);
 	void (*complexity_change) (GstDialog *);
 };
 
