@@ -51,7 +51,10 @@
 #include <sys/wait.h>
 #include <signal.h>
 
+#define BUFFER_SIZE 2048
 #define GST_DEBUG 1
+#define GST_TOOL_EOR "<!-- GST: end of request -->"
+
 /* Define this if you want the frontend to run the backend under strace */
 /*#define GST_DEBUG_STRACE_BACKEND*/
 
@@ -65,6 +68,7 @@ GdkPixbuf *suse;
 GdkPixbuf *freebsd;
 GdkPixbuf *gentoo;
 GdkPixbuf *pld;
+GdkPixbuf *openna;
 
 enum {
 	BOGUS,
@@ -84,7 +88,6 @@ enum {
 
 static GtkObjectClass *parent_class;
 static gint gsttool_signals [LAST_SIGNAL] = { 0 };
-static const gchar *GST_TOOL_EOR = "<!-- GST: end of request -->";
 
 static void     gst_tool_idle_run_directives_remove (GstTool *tool);
 static void     gst_tool_idle_run_directives_add    (GstTool *tool);
@@ -260,29 +263,6 @@ gst_tool_get_access (GstTool *tool)
 	return tool->root_access != ROOT_ACCESS_NONE;
 }
 
-#if 0
-static void
-timeout_cb (gpointer data)
-{
-	GstTool *tool;
-
-	tool = GST_TOOL (data);
-
-	tool->timeout_done = TRUE;
-
-	if (tool->timeout_done && !tool->report_list_visible)
-		gtk_main_quit ();
-}
-
-static void
-set_arrow (GstTool *tool, GtkArrowType arrow)
-{
-	gtk_notebook_set_page (GTK_NOTEBOOK (tool->report_notebook),
-			       arrow == GTK_ARROW_UP ? 0 : 1);
-}
-
-#endif
-
 static void
 report_clear_lines (GstTool *tool)
 {
@@ -307,23 +287,6 @@ report_dispatch (GstTool *tool)
 {
 	GSList *list;
 	GstReportLine *rline;
-#if 0
-	GtkAdjustment *vadj;
-	char *report_text[] = {
-		NULL,
-		NULL
-	};
-
-	if (!GTK_IS_SCROLLED_WINDOW (tool->report_scrolled)) {
-		g_warning ("tool->report_scrolled is not a GtkScrolledWindow\n");
-		tool->report_dispatch_pending = FALSE;
-		tool->report_finished = TRUE;
-		return;
-	}
-	
-	vadj = gtk_scrolled_window_get_vadjustment (
-		GTK_SCROLLED_WINDOW (tool->report_scrolled));
-#endif
 
 	for (list = tool->report_line_list; list; list = g_slist_next (list))
 	{
@@ -332,35 +295,6 @@ report_dispatch (GstTool *tool)
 		if (gst_report_line_get_handled (rline))
 			continue;
 
-#if 0
-		if (!strcmp (gst_report_line_get_key (rline), "progress")) {
-			/* Progress update */
-
-			gtk_progress_set_percentage (GTK_PROGRESS (tool->report_progress),
-						     atoi (gst_report_line_get_message (rline)) / 100.0);
-		} else {
-			gboolean scroll;
-			
-			/* Report line */
-			
-			gtk_entry_set_text (GTK_ENTRY (tool->report_entry),
-					    gst_report_line_get_message (rline));
-			
-			if (vadj->value >= vadj->upper - vadj->page_size)
-				scroll = TRUE;
-			else
-				scroll = FALSE;
-
-			report_text [1] = (char *) gst_report_line_get_message (rline);
-
-			gtk_clist_append (GTK_CLIST (tool->report_list), report_text);
-
-			if (scroll)
-				gtk_adjustment_set_value (vadj, vadj->upper - vadj->page_size);
-
-			gst_tool_invoke_report_hooks (tool, tool->report_hook_type, rline);
-		}
-#endif
 		if (strcmp (gst_report_line_get_key (rline), "progress"))
 			gst_tool_invoke_report_hooks (tool, tool->report_hook_type, rline);
 		
@@ -379,8 +313,8 @@ report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
 {
 	GstTool *tool;
 	GstReportLine *rline;
-	gchar buffer [500];
-	gint n, i = 0;
+	gchar *buffer;
+	gint i = 0;
 
 	tool = GST_TOOL (data);
 	if (tool->input_block)
@@ -389,11 +323,8 @@ report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
 	if (!tool->line)
 		tool->line = g_string_new ("");
 
-	while ((n = read (fd, buffer, sizeof (buffer) - 1)) != 0) {
-		buffer [n] = 0;
-		fflush (NULL);
-
-		for (i = 0; (i < n); i++) {
+	while ((buffer = gst_tool_read_from_backend (tool))) {
+		for (i = 0; (i < strlen (buffer)); i++) {
 			gchar c = buffer [i];
 
 			if (c == '\n') {
@@ -422,17 +353,11 @@ report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
 		}
 	}
 
- full_break:
-
-	if (n <= 0) {
-		/* Zero-length read; pipe closed unexpectedly */
-
-		tool->report_finished = TRUE;
-	}
-
+full_break:
 	if (tool->report_finished) {
-		if (n > 0)
+		if ((strlen (buffer) - i) > 0)
 			tool->xml_document = g_string_new (&buffer [i]);
+		
 		g_string_free (tool->line, TRUE);
 		tool->line = NULL;
 	}
@@ -458,9 +383,6 @@ report_window_close_cb (GtkWidget *window, GdkEventAny *ev, gpointer data)
 static void
 report_progress (GstTool *tool, const gchar *label)
 {
-#if 0
-	gint cb_id;
-#endif	
 	tool->timeout_done = FALSE;
 	tool->report_list_visible = FALSE;
 	tool->report_finished = FALSE;
@@ -469,11 +391,6 @@ report_progress (GstTool *tool, const gchar *label)
 	gst_tool_clear_supported_platforms (tool);
 	report_clear_lines (tool);
 
-#if 0
-	set_arrow (tool, GTK_ARROW_DOWN);
-	gtk_progress_set_percentage (GTK_PROGRESS (tool->report_progress), 0.0);
-#endif
-
 	if (label) {
 		gtk_label_set_text (GTK_LABEL (tool->report_label), label);
 		gtk_widget_show_all (tool->report_window);
@@ -492,24 +409,10 @@ report_progress (GstTool *tool, const gchar *label)
 					     report_progress_tick, NULL, tool, NULL);
 	tool->input_block = FALSE;
 
-	gtk_main (); 
+	gtk_main ();
 
 	if (tool->input_id)
 		gtk_input_remove (tool->input_id);
-
-#if 0
-	if (!tool->run_again)
-	{
-		/* Set progress to 100% and wait a bit before closing display */
-		gtk_progress_set_percentage (GTK_PROGRESS (tool->report_progress), 1.0);
-
-		cb_id = gtk_timeout_add (1500, (GtkFunction) timeout_cb, tool);
-		gtk_main ();
-		gtk_timeout_remove (cb_id);
-	}
-
-	gtk_clist_clear (GTK_CLIST (tool->report_list));
-#endif
 }
 
 static GSList *
@@ -548,6 +451,7 @@ gst_tool_create_distro_images (void)
 	freebsd = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/freebsd.png", NULL);
 	gentoo = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/gentoo.png", NULL);
 	pld = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/pld.png", NULL);
+	openna = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/openna.png", NULL);
 }
 
 static void
@@ -676,7 +580,10 @@ static void
 gst_tool_kill_backend_cb (GstDirectiveEntry *entry)
 {
 	GstTool *tool = entry->tool;
-	
+
+	/* removes the g_timeout that's waiting for the backend to die */
+	g_source_remove (tool->timeout_id);
+
 	/* The backend was never called. No problem! */
 	/* For further reference check http://www.xs4all.nl/~pjbrink/apekool/050/045-en.html */
 	if (tool->backend_pid > 0) {
@@ -744,38 +651,18 @@ gst_tool_end_of_request (GString *string)
 static xmlDoc *
 gst_tool_read_xml_from_backend (GstTool *tool)
 {
-	char buffer [4096];
+	gchar *buffer;
 	int t;
 	xmlDoc *xml;
 
 	if (!tool->xml_document)
 		return NULL;
 
-	fcntl (tool->backend_master_fd, F_SETFL, O_NONBLOCK);
-
 	while (! gst_tool_end_of_request (tool->xml_document)) {
-		while (gtk_events_pending ())
-			gtk_main_iteration ();
-		while (g_main_context_iteration (NULL, FALSE));
-
-		t = read (tool->backend_master_fd, buffer, sizeof (buffer) - 1);
-		buffer[t] = '\0';
-
-		fflush (NULL);
-
-		if ((t == 0))
-			break;
-		if (t == -1)
-			t = 0;
-
-		if (t > 0) {
-			buffer [t] = 0;
-			g_string_append (tool->xml_document, buffer);
-		}
+		buffer = gst_tool_read_from_backend (tool);
+		g_string_append (tool->xml_document, buffer);
 	}
 
-	fcntl (tool->backend_master_fd, F_SETFL, 0);
-
 	if (tool->xml_document->str[0] == '<') {
 		xml = xmlParseDoc (tool->xml_document->str);
 	} else {
@@ -854,21 +741,14 @@ gst_tool_send_directive (GstTool *tool, const gchar *directive, va_list ap)
 {
 	GString *directive_line;
 	FILE *f;
-	gchar buffer[512];
+	gchar *buffer;
 
 	g_return_if_fail (tool->backend_pid >= 0);
 
        	directive_line = gst_tool_join_directive (directive, ap);
 	g_string_append_c (directive_line, '\n');
 
-	f = fdopen (dup (tool->backend_master_fd), "w");
-	fprintf (f, directive_line->str);
-	fflush (f);
-	fclose (f);
-
-	/* clear the buffer from the directive, this is done because there is only one
-	 * fd for reading and writing */
-	read (tool->backend_master_fd, buffer, strlen (directive_line->str));
+	gst_tool_write_to_backend (tool, directive_line->str);
 
 	g_string_free (directive_line, TRUE);
 }
@@ -927,7 +807,6 @@ gst_tool_run_set_directive_va (GstTool *tool, xmlDoc *xml,
 {
 	FILE *f;
 	xmlDoc *xml_out;
-	gchar *eor = g_strconcat ("\n", GST_TOOL_EOR, "\n", NULL);
 
 	int n;
 	gchar buf;
@@ -949,21 +828,10 @@ gst_tool_run_set_directive_va (GstTool *tool, xmlDoc *xml,
 	gst_tool_send_directive (tool, directive, ap);
 
 	if (xml) {
-		f = fdopen (dup (tool->backend_master_fd), "w");
-		xmlDocDump (f, xml);
-
-		fflush (f);
-		fclose (f);
-
-		/* read all output from the fd, this is done because a single fd is user for
-		 * reading and writing */
-		fcntl (tool->backend_master_fd, F_SETFL, O_NONBLOCK);
-		do {
-			n = read (tool->backend_master_fd, &buf, 1);
-		} while (n >= 0);
-		fcntl (tool->backend_master_fd, F_SETFL, O_NONBLOCK);
-
-		write (tool->backend_master_fd, eor, strlen (eor));
+		gst_tool_write_xml_to_backend (tool, xml);
+		gst_tool_write_to_backend (tool, "\n");
+		gst_tool_write_to_backend (tool, GST_TOOL_EOR);
+		gst_tool_write_to_backend (tool, "\n");
 	}
 
 
@@ -1004,7 +872,9 @@ gst_tool_load (GstTool *tool)
   
 	if (tool->config) {
 		xmlFreeDoc (tool->config);
+		xmlFreeDoc (tool->original_config);
 		tool->config = NULL;
+		tool->original_config = NULL;
 	}
 
 	if (tool->run_again)
@@ -1012,6 +882,7 @@ gst_tool_load (GstTool *tool)
 
 	if (location_id == NULL) {
 		tool->config = gst_tool_run_get_directive (tool, NULL, "get", NULL);
+		tool->original_config = xmlCopyDoc (tool->config, 1);
 	} else {
 #ifndef GST_HAVE_ARCHIVER
 		g_assert_not_reached ();
@@ -1060,18 +931,8 @@ gst_tool_load (GstTool *tool)
 	return tool->config != NULL;
 }
 
-#if 0
-static void
-gst_tool_save_directive_callback (GstDirectiveEntry *entry)
-{
-	gst_tool_run_set_directive_va (entry->tool, entry->in_xml, entry->report_sign,
-				       entry->directive, entry->ap);
-	gst_dialog_thaw_visible (entry->tool->main_dialog);
-}
-#endif
-
 gboolean
-gst_tool_save (GstTool *tool)
+gst_tool_save (GstTool *tool, gboolean restore)
 {
 #ifdef GST_HAVE_ARCHIVER
 	CORBA_Environment ev;
@@ -1087,7 +948,6 @@ gst_tool_save (GstTool *tool)
 	gst_dialog_freeze_visible (tool->main_dialog);
 
 	g_signal_emit (GTK_OBJECT (tool), gsttool_signals[FILL_XML], 0);
-
 #ifdef GST_DEBUG
 	/* don't actually save if we are just pretending */
 	if (tool->root_access == ROOT_ACCESS_SIMULATED) {
@@ -1132,8 +992,13 @@ gst_tool_save (GstTool *tool)
 
 #endif
 	
-	if (location_id == NULL)
-		gst_tool_run_set_directive (tool, tool->config, NULL, "set", NULL);
+	if (location_id == NULL) {
+		if (restore)
+			gst_tool_run_set_directive (tool, tool->original_config, NULL, "set", NULL);
+		else
+			gst_tool_run_set_directive (tool, tool->config, NULL, "set", NULL);
+	}
+	
 	gst_dialog_thaw_visible (tool->main_dialog);
 
 	return TRUE;  /* FIXME: Determine if it really worked. */
@@ -1142,7 +1007,13 @@ gst_tool_save (GstTool *tool)
 void
 gst_tool_save_cb (GtkWidget *w, GstTool *tool)
 {
-	gst_tool_save (tool);
+	gst_tool_save (tool, FALSE);
+}
+
+void
+gst_tool_restore_cb (GtkWidget *w, GstTool *tool)
+{
+	gst_tool_save (tool, TRUE);
 }
 
 void
@@ -1163,11 +1034,13 @@ gst_tool_load_try (GstTool *tool)
 	}
 }
 
-void
-gst_tool_main (GstTool *tool, gboolean no_main_loop)
+static void
+tool_main_do (GstTool *tool, gboolean no_main_loop, gboolean show_main_dialog)
 {
 	gst_dialog_freeze_visible (tool->main_dialog);
-	gtk_widget_show (GTK_WIDGET (tool->main_dialog));
+
+	if (show_main_dialog)
+		gtk_widget_show (GTK_WIDGET (tool->main_dialog));
 	
 	if (tool->remote_config == TRUE) {
 		/* run the ssh client */
@@ -1176,7 +1049,6 @@ gst_tool_main (GstTool *tool, gboolean no_main_loop)
 	} else {
 		/* run the su command */
 		gst_dialog_freeze_visible (tool->main_dialog);
-		gtk_widget_show (GTK_WIDGET (tool->main_dialog));
 
 		gst_auth_do_su_authentication (tool);
 
@@ -1191,6 +1063,18 @@ gst_tool_main (GstTool *tool, gboolean no_main_loop)
 		gtk_main ();
 }
 
+void
+gst_tool_main (GstTool *tool, gboolean no_main_loop)
+{
+	tool_main_do (tool, no_main_loop, TRUE);
+}
+
+void
+gst_tool_main_with_hidden_dialog (GstTool *tool, gboolean no_main_loop)
+{
+	tool_main_do (tool, no_main_loop, FALSE);
+}
+
 static void
 gst_tool_destroy (GtkObject *object)
 {
@@ -1236,46 +1120,9 @@ gst_tool_class_init (GstToolClass *klass)
 			      gst_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
 
-#if 0	
-	gtk_object_class_add_signals (object_class, gsttool_signals, LAST_SIGNAL);
-#endif	
-
 	object_class->destroy = gst_tool_destroy;
 }
 
-#if 0
-
-static void
-visibility_toggled (GtkWidget *w, gpointer data)
-{
-	GstTool *tool;
-	gboolean show;
-	GtkAdjustment *vadj;
-
-	tool = GST_TOOL (data);
-
-	show = GTK_TOGGLE_BUTTON (w)->active;
-	vadj = gtk_scrolled_window_get_vadjustment (
-		GTK_SCROLLED_WINDOW (tool->report_scrolled));
-
-	set_arrow (tool, show ? GTK_ARROW_UP : GTK_ARROW_DOWN);
-
-	if (show) {
-		gtk_widget_set_usize (tool->report_scrolled, -1, 140);
-		gtk_widget_show (tool->report_scrolled);
-		gtk_adjustment_set_value (vadj, vadj->upper - vadj->page_size);
-		gtk_adjustment_value_changed (vadj);
-	} else { 
-		gtk_widget_hide (tool->report_scrolled);
-		if (tool->timeout_done)
-			gtk_main_quit();
-	}
-
-	tool->report_list_visible = show;
-}
-
-#endif
-
 static void
 gst_tool_create_platform_list (GtkTreeView *list, GstTool *tool)
 {
@@ -1336,9 +1183,6 @@ on_remote_hosts_list_selection_changed (GtkTreeSelection *selection, gpointer da
 			gst_dialog_ask_apply (tool->main_dialog);
 			gst_dialog_set_modified (tool->main_dialog, FALSE);
 
-			/* removes the g_timeout that's waiting for the backend to die */
-			g_source_remove (tool->timeout_id);
-
 			/* stops the backend */
 			gst_tool_kill_backend (tool, NULL);
 
@@ -1411,33 +1255,8 @@ gst_tool_type_init (GstTool *tool)
 	tool->report_window     = glade_xml_get_widget (xml, "report_window");
 	g_signal_connect (GTK_OBJECT (tool->report_window), "delete_event",
 			    G_CALLBACK (report_window_close_cb), tool);
-#if 0
-	tool->report_scrolled   = glade_xml_get_widget (xml, "report_list_scrolled_window");
-	tool->report_progress   = glade_xml_get_widget (xml, "report_progress");
-#endif
+
 	tool->report_label      = glade_xml_get_widget (xml, "report_label");
-#if 0
-	tool->report_list       = glade_xml_get_widget (xml, "report_list");
-	tool->report_entry      = glade_xml_get_widget (xml, "report_text");
-	tool->report_visibility = glade_xml_get_widget (xml, "report_visibility");
-	tool->report_notebook   = glade_xml_get_widget (xml, "report_notebook");
-
-	gtk_notebook_remove_page (GTK_NOTEBOOK (tool->report_notebook), 0);
-	gtk_notebook_append_page (GTK_NOTEBOOK (tool->report_notebook),
-				  gnome_stock_pixmap_widget_new (tool->report_notebook, 
-								 GNOME_STOCK_PIXMAP_UP),
-				  NULL);
-
-	gtk_notebook_append_page (GTK_NOTEBOOK (tool->report_notebook),
-				  gnome_stock_pixmap_widget_new (tool->report_notebook, 
-								 GNOME_STOCK_PIXMAP_DOWN),
-				  NULL);
-
-	set_arrow (tool, GTK_ARROW_DOWN);
-	gtk_widget_show_all (tool->report_notebook);
-
-	glade_xml_signal_connect_data (xml, "visibility_toggled", visibility_toggled, tool);
-#endif
 
 	xml = gst_tool_load_glade_common (tool, "platform_dialog");
 	tool->platform_list = glade_xml_get_widget (xml, "platform_list");
@@ -1521,6 +1340,10 @@ gst_tool_construct (GstTool *tool, const char *name, const char *title)
 			  "apply",
 			  G_CALLBACK (gst_tool_save_cb),
 			  tool);
+	g_signal_connect (G_OBJECT (tool->main_dialog),
+			  "restore",
+			  G_CALLBACK (gst_tool_restore_cb),
+			  tool);
 
 	tool->report_hook_list = NULL;
 	memset (&tool->report_hook_defaults, 0, sizeof (GstReportHook *) * GST_MAJOR_MAX);
@@ -1787,3 +1610,66 @@ gst_init (const gchar *app_name, int argc, char *argv [], const poptOption optio
 				      NULL);
 	try_show_usage_warning ();
 }
+
+gchar*
+gst_tool_read_from_backend (GstTool *tool)
+{
+	gchar *buffer;
+	GString *str = g_string_new ("");
+	gchar *ptr;
+	gboolean may_exit = FALSE;
+	
+	buffer = (gchar*) g_malloc0 (sizeof (gchar) * BUFFER_SIZE);
+
+	do {
+		usleep (32000);
+		
+		while (fgets (buffer, BUFFER_SIZE, tool->backend_stream) != NULL) {
+			g_string_append (str, buffer);
+			may_exit = TRUE;
+		}
+		
+		while (gtk_events_pending ())
+			gtk_main_iteration ();
+	} while (!may_exit);
+
+	ptr = str->str;
+	g_string_free (str, FALSE);
+
+	return ptr;
+}
+
+void
+gst_tool_write_xml_to_backend (GstTool *tool, xmlDoc *doc)
+{
+	gint size;
+	xmlChar *xml;
+	gchar *string;
+
+	xmlDocDumpMemory (doc, &xml, &size);
+	string = (gchar *) xml;
+	
+	gst_tool_write_to_backend (tool, string);
+
+	xmlFree (xml);
+
+	usleep (32000);
+}
+
+void
+gst_tool_write_to_backend (GstTool *tool, gchar *string)
+{
+	gint ntotal = 0;
+	gint nread = 0;
+	gint ret;
+	gchar *p;
+
+	do {
+		ret = fputc (string [nread], tool->backend_stream);
+
+		if (ret >= 0)
+			nread++;
+	} while (nread < strlen (string));
+
+	while (fflush (tool->backend_stream) != 0);
+}
