@@ -50,7 +50,7 @@ GstDialogSignal signals[] = {
 	{ "connection_configure",    "clicked",         G_CALLBACK (on_connection_configure_clicked) },
 	{ "connection_def_gw_omenu", "changed",         G_CALLBACK (gst_dialog_modify_cb) },
 	{ "domain",                  "changed",         G_CALLBACK (gst_dialog_modify_cb) },
-	{ "ip",                      "changed",         G_CALLBACK (gst_dialog_modify_cb) },
+/*	{ "ip",                      "changed",         G_CALLBACK (gst_dialog_modify_cb) },*/
 	{ "ip",                      "changed",         G_CALLBACK (on_hosts_ip_changed) },
 	{ "statichost_add",          "clicked",         G_CALLBACK (on_hosts_add_clicked) },
 	{ "statichost_add",          "clicked",         G_CALLBACK (gst_dialog_modify_cb) },
@@ -214,6 +214,7 @@ main (int argc, char *argv[])
 	gboolean wlan_connection = FALSE;
 	gboolean plip_connection = FALSE;
 	gboolean irlan_connection = FALSE;
+	gchar *interface = NULL;
 	
 	struct poptOption options[] =
 	{
@@ -223,6 +224,7 @@ main (int argc, char *argv[])
 		{ "wlan-connection", '\0', 0, &wlan_connection, 0, _("Run the wlan connection druid."), NULL },
 		{ "plip-connection", '\0', 0, &plip_connection, 0, _("Run the parallel port connection druid."), NULL },
 		{ "irlan-connection", '\0', 0, &irlan_connection, 0, _("Run the irlan connection druid."), NULL },
+		{ "configure", '\0', POPT_ARG_STRING, &interface, 0, _("Configure a network interface"), _("interface") },
 		
 		{NULL, '\0', 0, NULL, 0}
 	};
@@ -250,8 +252,6 @@ main (int argc, char *argv[])
 		else if (irlan_connection)
 			type = GST_CONNECTION_IRLAN;
 
-		gst_tool_load_try (tool);
-
 		g_object_set_data (G_OBJECT (druid), "standalone", GINT_TO_POINTER (TRUE));
 
 		connect_signals (tool->main_dialog, signals, signals_after);
@@ -259,6 +259,23 @@ main (int argc, char *argv[])
 		network_druid_new (druid, tool, type);
 		gtk_widget_show_all (druid_window);
 
+		gst_tool_main_with_hidden_dialog (tool, FALSE);
+	} else if (interface != NULL) {
+		GtkWidget *window = gst_dialog_get_widget (tool->main_dialog, "connection_config_dialog");
+		GstConnection *connection;
+		xmlNodePtr root;
+
+		connect_signals (tool->main_dialog, signals, signals_after);
+		init_editable_filters (tool->main_dialog);
+		gst_dialog_set_widget_policies (tool->main_dialog, policies);
+
+		gst_tool_main_with_hidden_dialog (tool, TRUE);
+
+		g_object_set_data (G_OBJECT (window), "standalone", GINT_TO_POINTER (TRUE));
+		root = gst_xml_doc_get_root (tool->config);
+		transfer_misc_xml_to_tool (tool, root);
+		connection_configure_device (root, interface);
+
 		gtk_main ();
 	} else {
 		connect_signals (tool->main_dialog, signals, signals_after);
@@ -277,7 +294,15 @@ main (int argc, char *argv[])
 
 		on_network_admin_show (NULL, tool);
 		update_complexity (tool->main_dialog, NULL);
-		gst_tool_main (tool, FALSE);
+
+		gst_tool_main (tool, TRUE);
+		
+		if (connection_get_count (tool) == 0)
+			g_signal_emit_by_name (G_OBJECT (gst_dialog_get_widget (tool->main_dialog, "connection_add")),
+					       "clicked",
+					       NULL);
+
+		gtk_main ();
 	}
 		
 	return 0;
